"""
Toggles for learner recommendations.
"""
from edx_toggles.toggles import WaffleFlag

# Namespace for edx_recommendations waffle flags.
WAFFLE_FLAG_NAMESPACE = "edx_recommendations"

# Waffle flag to enable to recommendation panel on learner dashboard
# .. toggle_name: edx_recommendations.enable_dashboard_recommendations
# .. toggle_implementation: WaffleFlag
# .. toggle_default: False
# .. toggle_description: Waffle flag to enable to recommendation panel on learner dashboard
# .. toggle_use_cases: temporary
# .. toggle_creation_date: 2023-03-24
# .. toggle_target_removal_date: None
# .. toggle_warning: None
# .. toggle_tickets: VAN-1310
ENABLE_DASHBOARD_RECOMMENDATIONS = WaffleFlag(
    f"{WAFFLE_FLAG_NAMESPACE}.enable_dashboard_recommendations", __name__
)

# Waffle flag to enable fallback recommendations.
# .. toggle_name: edx_recommendations.enable_fallback_recommendations
# .. toggle_implementation: WaffleFlag
# .. toggle_default: False
# .. toggle_description: Supports showing fallback recommendation in case of error on amplitude side.
#                        Currently, fallback recommendations are picked from settings.GENERAL_RECOMMENDATIONS.
# .. toggle_use_cases: opt_in
# .. toggle_creation_date: 2023-01-16
# .. toggle_target_removal_date: None
# .. toggle_warning: None
# .. toggle_tickets: VAN-1239
FALLBACK_RECOMMENDATIONS = WaffleFlag(
    f"{WAFFLE_FLAG_NAMESPACE}.enable_fallback_recommendations", __name__
)



"""
Tests for helpers methods
"""
from unittest.mock import Mock, patch

import ddt
from django.test import TestCase

from edx_recommendations.api.utils import (
    _has_country_restrictions,
    filter_recommended_courses,
    get_active_course_run,
    get_amplitude_course_recommendations,
    get_cross_product_recommendations,
)
from test_utils.constants import mock_cross_product_recommendation_keys
from test_utils.factories import CourseEnrollmentFactory, UserFactory


@ddt.ddt
class TestRecommendationsHelper(TestCase):
    """Test course recommendations helper methods."""

    def setUp(self):
        super().setUp()
        self.user = UserFactory()

    @ddt.data(
        ({}, 0),
        ({"userData": {}}, 0),
        ({"userData": {"recommendations": []}}, 0),
        (
            {
                "userData": {
                    "recommendations": [
                        {
                            "items": ["MITx+6.00.1x", "IBM+PY0101EN", "HarvardX+CS50P"],
                            "is_control": True,
                            "has_is_control": False,
                        }
                    ],
                }
            },
            3,
        ),
    )
    @patch("edx_recommendations.api.utils.requests.get")
    @ddt.unpack
    def test_get_amplitude_course_recommendations_method(
        self, mocked_response, expected_recommendations_count, mock_get
    ):
        """
        Tests the get_amplitude_recommendations method returns course key list.
        """
        mock_get.return_value = Mock(status_code=200, json=lambda: mocked_response)
        _, _, course_keys = get_amplitude_course_recommendations(
            self.user.id, "amplitude-rec-id"
        )
        self.assertEqual(len(course_keys), expected_recommendations_count)

    @ddt.data(
        ({}, False),
        ({"restriction_type": "blocklist", "countries": []}, False),
        ({"restriction_type": "blocklist", "countries": ["SA"]}, False),
        ({"restriction_type": "blocklist", "countries": ["US"]}, True),
        ({"restriction_type": "allowlist", "countries": []}, False),
        ({"restriction_type": "allowlist", "countries": ["SA"]}, True),
        ({"restriction_type": "allowlist", "countries": ["US"]}, False),
    )
    @ddt.unpack
    def test_has_country_restrictions_method(
        self,
        location_restriction,
        expected_response,
    ):
        """
        Helper method to test the _has_country_restrictions method.
        """
        product = {"location_restriction": location_restriction}
        assert _has_country_restrictions(product, "US") == expected_response


class TestFilterRecommendedCourses(TestCase):
    """Test for filter_recommended_courses helper method."""

    def setUp(self):
        super().setUp()
        self.user = UserFactory()
        self.recommended_course_keys = [
            "MITx+6.00.1x",
            "IBM+PY0101EN",
            "HarvardX+CS50P",
            "UQx+IELTSx",
            "HarvardX+CS50x",
            "Harvard+CS50z",
            "BabsonX+EPS03x",
            "TUMx+QPLS2x",
            "NYUx+FCS.NET.1",
            "MichinX+101x",
        ]
        self.unrestricted_course_keys = self.recommended_course_keys[0:2]
        self.course_run_keys = [f"course-v1:{course_key}+Run_0" for course_key in self.recommended_course_keys]
        self.course_keys_with_active_course_runs = self.recommended_course_keys[0:8]
        self.enrolled_course_run_keys = self.course_run_keys[4:10]

    def _mock_get_course_data(self, course_id, fields=None, querystring=None):  # pylint: disable=unused-argument
        """
        Mocked response for the get_course_data call
        """
        course_data = {
            "course_key": course_id,
            "title": "Mocked course title",
            "owners": [{"logo_image_url": "https://www.logo_image_url.com"}],
            "marketing_url": "https://www.marketing_url.com",
        }

        if course_id not in self.unrestricted_course_keys:
            course_data.update(
                {
                    "location_restriction": {
                        "restriction_type": "blocklist",
                        "countries": ["US"],
                    }
                }
            )

        if course_id in self.course_keys_with_active_course_runs:
            course_data.update(
                {
                    "course_runs": [
                        {
                            "key": f"course-v1:{course_id}+Run_0",
                        }
                    ]
                }
            )

        return course_data

    @patch("edx_recommendations.api.utils.get_course_data")
    def test_enrolled_courses_are_removed_from_recommendations(
        self, mocked_get_course_data
    ):
        """
        Tests that given a recommended course list, the filter_recommended_courses
        method removes the enrolled courses from it.
        """
        total_enrolled_courses = len(self.enrolled_course_run_keys)
        total_recommendations = len(self.recommended_course_keys)
        mocked_get_course_data.side_effect = self._mock_get_course_data
        for course_run_key in self.enrolled_course_run_keys:
            CourseEnrollmentFactory(course_id=course_run_key, user=self.user)

        filtered_courses = filter_recommended_courses(
            self.user, self.recommended_course_keys, total_recommendations
        )
        assert len(filtered_courses) == (total_recommendations - total_enrolled_courses)

    @patch("edx_recommendations.api.utils.get_course_data")
    def test_request_course_is_removed_from_the_recommendations(
        self,
        mocked_get_course_data,
    ):
        """
        Test that if the "request course" is one of the recommended courses,
        we filter that from the final recommendation list.
        """
        request_course = self.course_run_keys[0]
        mocked_get_course_data.side_effect = self._mock_get_course_data
        filtered_courses = filter_recommended_courses(
            self.user,
            self.recommended_course_keys,
            request_course_key=request_course,
        )

        assert all(course["course_key"] != request_course for course in filtered_courses) is True

    @patch("edx_recommendations.api.utils.get_course_data")
    def test_country_restrictions_for_the_recommended_course(
        self,
        mocked_get_course_data,
    ):
        """
        Test that if a recommended course is restricted in the country the user
        is logged from, the course is filtered out.
        """
        mocked_get_course_data.side_effect = self._mock_get_course_data
        filtered_courses = filter_recommended_courses(
            self.user, self.recommended_course_keys, user_country_code="US"
        )
        expected_recommendations = []
        for course_key in self.unrestricted_course_keys:
            expected_recommendations.append(self._mock_get_course_data(course_key))

        assert filtered_courses == expected_recommendations

    @patch("edx_recommendations.api.utils.get_course_data")
    def test_recommend_only_active_courses(
        self,
        mocked_get_course_data,
    ):
        """
        Test that courses having no active course runs are filtered out from recommended courses.
        """
        mocked_get_course_data.side_effect = self._mock_get_course_data
        filtered_courses = filter_recommended_courses(
            self.user, self.recommended_course_keys
        )
        expected_recommendations = []
        for course_key in self.course_keys_with_active_course_runs:
            expected_recommendations.append(self._mock_get_course_data(course_key))

        assert filtered_courses == expected_recommendations


@ddt.ddt
class TestGetCrossProductRecommendationsMethod(TestCase):
    """Test for get_cross_product_recommendations method"""

    @ddt.data(
        ("edx+HL0", ["edx+HL1", "edx+HL2"]),
        ("edx+BZ0", ["edx+BZ1", "edx+BZ2"]),
        ('NoKeyAssociated', None)
    )
    @patch("django.conf.settings.CROSS_PRODUCT_RECOMMENDATIONS_KEYS", mock_cross_product_recommendation_keys)
    @ddt.unpack
    def test_get_cross_product_recommendations_method(self, course_key, expected_response):
        assert get_cross_product_recommendations(course_key) == expected_response


class TestGetActiveCourseRunMethod(TestCase):
    """Tests for get_active_course_run method"""

    advertised_course_run_uuid = "jh76b2c9-589b-4d1e-88c1-b01a02db3a9c"

    def _mock_get_course_data(self, active_course_run=False):
        """
        Returns a course with details based on the status passed
        """
        return {
            "key": "edx+BLN",
            "uuid": "6f8cb2c9-589b-4d1e-88c1-b01a02db3a9c",
            "course_runs": [
                {
                    "key": "course-v1:Test+2023_T1",
                    "uuid": "hb86b3cf-589b-4d1e-88c1-b01a02db3a9c",
                    "status": "published",
                },
                {
                    "key": "course-v1:Test+2023_T2",
                    "uuid": self.advertised_course_run_uuid if active_course_run else "other-uuid",
                    "status": "published",
                }
            ],
            "course_run_statuses": ["published"],
            "advertised_course_run_uuid": self.advertised_course_run_uuid if active_course_run else None,
        }

    def test_advertised_course_run_returned(self):
        """
        Test that the course run with the uuid that matches the advertised_uuid_course_run_uuid is returned
        """
        course = self._mock_get_course_data(active_course_run=True)
        active_course_run = get_active_course_run(course)

        self.assertDictEqual(
            active_course_run,
            {
                "key": "course-v1:Test+2023_T2",
                "uuid": self.advertised_course_run_uuid,
                "status": "published"
            }
        )

    def test_no_course_run_returned(self):
        """
        Test that if there is no advertised_course_run_uuid value, no course run is returned
        """
        course = self._mock_get_course_data(active_course_run=False)
        active_course_run = get_active_course_run(course)

        self.assertIsNone(active_course_run)



"""
Tests for serializers
"""

from uuid import uuid4

from django.test import TestCase

from edx_recommendations.api.serializers import (
    AmplitudeRecommendationsSerializer,
    CrossProductAndAmplitudeRecommendationsSerializer,
    CrossProductRecommendationsSerializer,
    DashboardRecommendationsSerializer,
    RecommendationsContextSerializer,
)
from test_utils.constants import (
    mock_amplitude_and_cross_product_course_data,
    mock_amplitude_course_data,
    mock_cross_product_course_data,
)


class TestDashboardRecommendationsSerializer(TestCase):
    """
    Tests for dashboard recommendations serializer
    """

    @classmethod
    def mock_recommended_courses(cls, courses_count=2):
        """
        Mocked course data
        """

        recommended_courses = []

        for _ in range(courses_count):
            recommended_courses.append(
                {
                    "course_key": str(uuid4()),
                    "logo_image_url": "http://edx.org/images/test.png",
                    "marketing_url": "http://edx.org/courses/AI",
                    "title": str(uuid4()),
                },
            )

        return recommended_courses

    def test_no_recommended_courses(self):
        """
        Verify that data serializes correctly for empty course list
        """

        recommended_courses = self.mock_recommended_courses(courses_count=0)

        output_data = DashboardRecommendationsSerializer(
            {
                "courses": recommended_courses,
            }
        ).data

        self.assertDictEqual(
            output_data,
            {
                "courses": [],
                "isControl": None,
            },
        )

    def test_successful_serialization(self):
        """
        Test that context data serializes correctly
        """

        recommended_courses = self.mock_recommended_courses()

        output_data = DashboardRecommendationsSerializer(
            {
                "courses": recommended_courses,
                "is_control": False,
            }
        ).data

        self.assertDictEqual(
            output_data,
            {
                "courses": [
                    {
                        "courseKey": recommended_courses[0]["course_key"],
                        "logoImageUrl": recommended_courses[0]["logo_image_url"],
                        "marketingUrl": recommended_courses[0]["marketing_url"],
                        "title": recommended_courses[0]["title"],
                    },
                    {
                        "courseKey": recommended_courses[1]["course_key"],
                        "logoImageUrl": recommended_courses[1]["logo_image_url"],
                        "marketingUrl": recommended_courses[1]["marketing_url"],
                        "title": recommended_courses[1]["title"],
                    },
                ],
                "isControl": False,
            },
        )


class TestRecommendationsContextSerializer(TestCase):
    """
    Tests for recommendations context serializer
    """

    def test_successful_serialization(self):
        """
        Test that context data serializes correctly
        """

        serialized_data = RecommendationsContextSerializer(
            {
                "countryCode": "US",
            }
        ).data

        self.assertDictEqual(
            serialized_data,
            {
                "countryCode": "US",
            },
        )

    def test_empty_response_serialization(self):
        """
        Test that an empty response serializes correctly
        """

        serialized_data = RecommendationsContextSerializer(
            {
                "countryCode": "",
            }
        ).data

        self.assertDictEqual(
            serialized_data,
            {
                "countryCode": "",
            },
        )


class TestCrossProductRecommendationsSerializers(TestCase):
    """
    Tests for the
     - Cross Product Recommendations Serializer,
     - Amplitude Recommendations Serializer and
     - Cross Product And Amplitude Recommendations Serializer
    """

    def mock_recommended_courses(self, num_of_courses=2):
        """
        Mocked course data
        """

        recommended_courses = []

        for index in range(num_of_courses):
            recommended_courses.append(
                {
                    "key": f"edx+HL{index}",
                    "uuid": f"{index}f8cb2c9-589b-4d1e-88c1-b01a02db3a9c",
                    "title": f"Title {index}",
                    "image": {
                        "src": f"https://www.logo_image_url{index}.com",
                    },
                    "url_slug": f"https://www.marketing_url{index}.com",
                    "course_type": "executive-education",
                    "owners": [
                        {
                            "key": f"org-{index}",
                            "name": f"org {index}",
                            "logo_image_url": f"https://discovery.com/organization/logos/org-{index}.png",
                        },
                    ],
                    "course_runs": [
                        {
                            "key": f"course-v1:Test+2023_T{index}",
                            "marketing_url": f"https://www.marketing_url{index}.com",
                            "availability": "Current",
                        }
                    ],
                    "active_course_run": {
                        "key": f"course-v1:Test+2023_T{index}",
                        "marketing_url": f"https://www.marketing_url{index}.com",
                        "availability": "Current",
                    },
                    "active_course_run_key": f"course-v1:Test+2023_T{index}",
                    "marketing_url": f"https://www.marketing_url{index}.com",
                    "location_restriction": None
                },
            )

        return recommended_courses

    def test_successful_cross_product_recommendation_serialization(self):
        """
        Test that course data serializes correctly for CrossProductRecommendationSerializer
        """
        courses = self.mock_recommended_courses(num_of_courses=2)

        serialized_data = CrossProductRecommendationsSerializer({
            "courses": courses,
        }).data

        self.assertDictEqual(
            serialized_data,
            mock_cross_product_course_data
        )

    def test_successful_amplitude_recommendations_serialization(self):
        """
        Test the course data serializes correctly for AmplitudeRecommendationsSerializer
        """
        courses = self.mock_recommended_courses(num_of_courses=4)

        serialized_data = AmplitudeRecommendationsSerializer({
            "amplitudeCourses": courses
        }).data

        self.assertDictEqual(
            serialized_data,
            mock_amplitude_course_data
        )

    def test_successful_cross_product_and_amplitude_recommendations_serializer(self):
        """
        Test that course data serializes correctly for CrossProductAndAmplitudeRecommendationSerializer
        """

        cross_product_courses = self.mock_recommended_courses(num_of_courses=2)
        amplitude_courses = self.mock_recommended_courses(num_of_courses=4)

        serialized_data = CrossProductAndAmplitudeRecommendationsSerializer({
            "crossProductCourses": cross_product_courses,
            "amplitudeCourses": amplitude_courses,
        }).data

        self.assertDictEqual(
            serialized_data,
            mock_amplitude_and_cross_product_course_data
        )

    def test_no_cross_product_course_serialization(self):
        """
        Tests that empty course data for CrossProductRecommendationsSerializer serializes properly
        """

        serialized_data = CrossProductRecommendationsSerializer({
            "courses": [],
        }).data

        self.assertDictEqual(
            serialized_data,
            {
                "courses": [],
            },
        )

    def test_no_amplitude_courses_serialization(self):
        """
        Tests that empty course data for AmplitudeRecommendationsSerializer serializes properly
        """

        serialized_data = AmplitudeRecommendationsSerializer({
            "amplitudeCourses": [],
        }).data

        self.assertDictEqual(
            serialized_data,
            {
                "amplitudeCourses": [],
            },
        )

    def test_no_amplitude_and_cross_product_and_course_serialization(self):
        """
        Tests that empty course data for CrossProductRecommendationsSerializer serializes properly
        """

        serialized_data = CrossProductAndAmplitudeRecommendationsSerializer({
            "crossProductCourses": [],
            "amplitudeCourses": []
        }).data

        self.assertDictEqual(
            serialized_data,
            {
                "crossProductCourses": [],
                "amplitudeCourses": []
            },
        )



"""
Tests for course recommendations APIs
"""

import json
from unittest import mock

import ddt
from django.urls import reverse_lazy
from edx_toggles.toggles.testutils import override_waffle_flag
from rest_framework.test import APITestCase

from edx_recommendations.toggles import ENABLE_DASHBOARD_RECOMMENDATIONS, FALLBACK_RECOMMENDATIONS
from test_utils.factories import UserFactory


class TestRecommendationsBase(APITestCase):
    """Recommendations test base class"""

    def setUp(self):
        super().setUp()
        self.user = UserFactory()
        self.client.login(username=self.user.username, password="test")
        self.recommended_courses = [
            "MITx+6.00.1x",
            "IBM+PY0101EN",
            "HarvardX+CS50P",
            "UQx+IELTSx",
            "HarvardX+CS50x",
            "Harvard+CS50z",
            "BabsonX+EPS03x",
            "TUMx+QPLS2x",
            "NYUx+FCS.NET.1",
            "MichinX+101x",
        ]


@ddt.ddt
class TestDashboardRecommendationsApiView(TestRecommendationsBase):
    """Unit tests for the course recommendations on learner home page."""

    url = reverse_lazy("learner_dashboard_amplitude")

    GENERAL_RECOMMENDATIONS = [
        {
            "course_key": "HogwartsX+6.00.1x",
            "logo_image_url": "http://edx.org/images/test.png",
            "marketing_url": "http://edx.org/courses/AI",
            "title": "Defense Against the Dark Arts",
        },
        {
            "course_key": "MonstersX+SC101EN",
            "logo_image_url": "http://edx.org/images/test.png",
            "marketing_url": "http://edx.org/courses/AI",
            "title": "Scaring 101",
        },
    ]

    SERIALIZED_GENERAL_RECOMMENDATIONS = [
        {
            "courseKey": GENERAL_RECOMMENDATIONS[0]["course_key"],
            "logoImageUrl": GENERAL_RECOMMENDATIONS[0]["logo_image_url"],
            "marketingUrl": GENERAL_RECOMMENDATIONS[0]["marketing_url"],
            "title": GENERAL_RECOMMENDATIONS[0]["title"],
        },
        {
            "courseKey": GENERAL_RECOMMENDATIONS[1]["course_key"],
            "logoImageUrl": GENERAL_RECOMMENDATIONS[1]["logo_image_url"],
            "marketingUrl": GENERAL_RECOMMENDATIONS[1]["marketing_url"],
            "title": GENERAL_RECOMMENDATIONS[1]["title"],
        },
    ]

    def setUp(self):
        super().setUp()
        self.course_run_keys = [f"course-v1:{course}+Run_0" for course in self.recommended_courses]

    def _get_filtered_courses(self):
        """
        Returns the filtered course data
        """
        filtered_course = []
        for course_key in self.recommended_courses[:5]:
            filtered_course.append({
                "key": course_key,
                "title": f"Title for {course_key}",
                "logo_image_url": "https://www.logo_image_url.com",
                "marketing_url": "https://www.marketing_url.com",
            })

        return filtered_course

    @override_waffle_flag(ENABLE_DASHBOARD_RECOMMENDATIONS, active=False)
    def test_waffle_flag_off(self):
        """
        Verify API returns 404 if waffle flag is off.
        """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 404)
        self.assertEqual(response.data, None)

    @override_waffle_flag(FALLBACK_RECOMMENDATIONS, active=True)
    @override_waffle_flag(ENABLE_DASHBOARD_RECOMMENDATIONS, active=True)
    @mock.patch("django.conf.settings.GENERAL_RECOMMENDATIONS", GENERAL_RECOMMENDATIONS)
    @mock.patch(
        "edx_recommendations.api.course_recommendations.get_amplitude_course_recommendations"
    )
    def test_no_recommendations_from_amplitude(
        self, get_amplitude_course_recommendations_mock
    ):
        """
        Verify API returns general recommendations if no course recommendations from amplitude.
        """
        get_amplitude_course_recommendations_mock.return_value = [False, True, []]

        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

        response_content = json.loads(response.content)
        self.assertEqual(response_content.get("isControl"), False)
        self.assertEqual(
            response_content.get("courses"),
            self.SERIALIZED_GENERAL_RECOMMENDATIONS,
        )

    @override_waffle_flag(FALLBACK_RECOMMENDATIONS, active=True)
    @override_waffle_flag(ENABLE_DASHBOARD_RECOMMENDATIONS, active=True)
    @mock.patch("django.conf.settings.GENERAL_RECOMMENDATIONS", GENERAL_RECOMMENDATIONS)
    @mock.patch(
        "edx_recommendations.api.course_recommendations.get_amplitude_course_recommendations",
        mock.Mock(side_effect=Exception),
    )
    def test_amplitude_api_unexpected_error(self):
        """
        Test that if the Amplitude API gives an unexpected error, general recommendations are returned.
        """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

        response_content = json.loads(response.content)
        self.assertEqual(response_content.get("isControl"), None)
        self.assertEqual(
            response_content.get("courses"),
            self.SERIALIZED_GENERAL_RECOMMENDATIONS,
        )

    @override_waffle_flag(ENABLE_DASHBOARD_RECOMMENDATIONS, active=True)
    @mock.patch("edx_recommendations.api.course_recommendations.get_amplitude_course_recommendations")
    @mock.patch("edx_recommendations.api.course_recommendations.filter_recommended_courses")
    def test_amplitude_course_recommendations(
        self, filter_recommended_courses_mock, get_amplitude_course_recommendations_mock
    ):
        """
        Verify API returns course recommendations.
        """
        get_amplitude_course_recommendations_mock.return_value = [
            False,
            True,
            self.recommended_courses,
        ]

        filter_recommended_courses_mock.return_value = self._get_filtered_courses()
        expected_recommendations_length = 5

        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

        response_content = json.loads(response.content)
        self.assertEqual(response_content.get("isControl"), False)
        self.assertEqual(
            len(response_content.get("courses")), expected_recommendations_length
        )

    @override_waffle_flag(FALLBACK_RECOMMENDATIONS, active=True)
    @override_waffle_flag(ENABLE_DASHBOARD_RECOMMENDATIONS, active=True)
    @mock.patch("django.conf.settings.GENERAL_RECOMMENDATIONS", GENERAL_RECOMMENDATIONS)
    @mock.patch(
        "edx_recommendations.api.course_recommendations.get_amplitude_course_recommendations"
    )
    def test_dashboard_general_recommendations(
        self, get_amplitude_course_recommendations_mock
    ):
        """
        Test that a user gets general recommendations for the control group.
        """
        get_amplitude_course_recommendations_mock.return_value = [
            True,
            True,
            self.recommended_courses,
        ]

        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

        response_content = json.loads(response.content)
        self.assertEqual(response_content.get("isControl"), True)
        self.assertEqual(
            response_content.get("courses"),
            self.SERIALIZED_GENERAL_RECOMMENDATIONS,
        )

    @override_waffle_flag(FALLBACK_RECOMMENDATIONS, active=False)
    @override_waffle_flag(ENABLE_DASHBOARD_RECOMMENDATIONS, active=True)
    @mock.patch("django.conf.settings.GENERAL_RECOMMENDATIONS", GENERAL_RECOMMENDATIONS)
    @mock.patch(
        "edx_recommendations.api.course_recommendations.get_amplitude_course_recommendations"
    )
    def test_fallback_recommendations_disabled(
        self, get_amplitude_course_recommendations_mock
    ):
        """
        Test that a user gets no recommendations for the control group.
        """
        get_amplitude_course_recommendations_mock.return_value = [
            True,
            True,
            [],
        ]

        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

        response_content = json.loads(response.content)
        self.assertEqual(response_content.get("isControl"), True)
        self.assertEqual(response_content.get("courses"), [])

    @override_waffle_flag(FALLBACK_RECOMMENDATIONS, active=True)
    @override_waffle_flag(ENABLE_DASHBOARD_RECOMMENDATIONS, active=True)
    @mock.patch("django.conf.settings.GENERAL_RECOMMENDATIONS", GENERAL_RECOMMENDATIONS)
    @mock.patch(
        "edx_recommendations.api.course_recommendations.get_amplitude_course_recommendations"
    )
    @mock.patch("edx_recommendations.api.course_recommendations.filter_recommended_courses")
    def test_no_recommended_courses_after_filtration(
        self, filter_recommended_courses_mock, get_amplitude_course_recommendations_mock
    ):
        """
        Test that if after filtering already enrolled courses from Amplitude recommendations
        we are left with zero personalized recommendations, we return general recommendations.
        """
        filter_recommended_courses_mock.return_value = []
        get_amplitude_course_recommendations_mock.return_value = [
            False,
            True,
            self.recommended_courses,
        ]

        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

        response_content = json.loads(response.content)
        self.assertEqual(response_content.get("isControl"), False)
        self.assertEqual(
            response_content.get("courses"),
            self.SERIALIZED_GENERAL_RECOMMENDATIONS,
        )

    @ddt.data(
        (True, False, None),
        (False, True, False),
        (False, False, None),
        (True, True, True),
    )
    @mock.patch("edx_recommendations.api.course_recommendations.segment.track")
    @mock.patch("edx_recommendations.api.course_recommendations.filter_recommended_courses")
    @mock.patch(
        "edx_recommendations.api.course_recommendations.get_amplitude_course_recommendations"
    )
    @override_waffle_flag(ENABLE_DASHBOARD_RECOMMENDATIONS, active=True)
    @ddt.unpack
    def test_recommendations_viewed_segment_event(
        self,
        is_control,
        has_is_control,
        expected_is_control,
        get_amplitude_course_recommendations_mock,
        filter_recommended_courses_mock,
        segment_track_mock
    ):
        """
        Test that Segment event is emitted with desired properties.
        """
        get_amplitude_course_recommendations_mock.return_value = [
            is_control,
            has_is_control,
            self.recommended_courses,
        ]
        filter_recommended_courses_mock.return_value = self._get_filtered_courses()
        self.client.get(self.url)

        if has_is_control and not is_control:
            assert segment_track_mock.call_count == 1
        else:
            assert segment_track_mock.call_count == 2
        assert segment_track_mock.call_args[0][1] == "edx.bi.user.recommendations.viewed"
        self.assertEqual(segment_track_mock.call_args[0][2]["is_control"], expected_is_control)

    @mock.patch("edx_recommendations.api.course_recommendations.segment.track")
    @mock.patch("edx_recommendations.api.course_recommendations.filter_recommended_courses")
    @mock.patch(
        "edx_recommendations.api.course_recommendations.get_amplitude_course_recommendations"
    )
    @override_waffle_flag(ENABLE_DASHBOARD_RECOMMENDATIONS, active=True)
    @ddt.unpack
    def test_recommendations_unavailable_segment_event(
        self,
        get_amplitude_course_recommendations_mock,
        filter_recommended_courses_mock,
        segment_track_mock
    ):
        """
        Test that Segment event is emitted with desired properties.
        """
        get_amplitude_course_recommendations_mock.return_value = [
            False,
            True,
            [],
        ]
        filter_recommended_courses_mock.return_value = self._get_filtered_courses()
        self.client.get(self.url)

        assert segment_track_mock.call_count == 2

    @override_waffle_flag(ENABLE_DASHBOARD_RECOMMENDATIONS, active=True)
    @mock.patch(
        "edx_recommendations.api.course_recommendations.is_user_enrolled_in_ut_austin_masters_program"
    )
    def test_no_recommendations_for_masters_program_learners(
        self, is_user_enrolled_in_ut_austin_masters_program_mock
    ):
        """
        Verify API returns no recommendations if a user is enrolled in UT Austin masters program.
        """
        is_user_enrolled_in_ut_austin_masters_program_mock.return_value = True

        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

        response_content = json.loads(response.content)
        self.assertEqual(response_content.get("isControl"), None)
        self.assertEqual(response_content.get("courses"), [])



"""
edx_recommendations Django application initialization.
"""

from django.apps import AppConfig
from edx_django_utils.plugins.constants import PluginSettings, PluginURLs


class EdxRecommendationsConfig(AppConfig):
    """
    Configuration for the edx_recommendations Django application.
    """

    name = "edx_recommendations"
    label = "edx_recommendations"

    plugin_app = {
        PluginURLs.CONFIG: {
            "lms.djangoapp": {
                PluginURLs.NAMESPACE: "edx_recommendations",
                PluginURLs.APP_NAME: "edx_recommendations",
                PluginURLs.REGEX: "api/edx_recommendations/",
                PluginURLs.RELATIVE_PATH: "api.urls",
            }
        },
        PluginSettings.CONFIG: {
            "lms.djangoapp": {
                "common": {
                    PluginSettings.RELATIVE_PATH: "settings.common",
                },
                'devstack': {
                    PluginSettings.RELATIVE_PATH: 'settings.devstack',
                },
                "production": {
                    PluginSettings.RELATIVE_PATH: "settings.production",
                },
            }
        },
    }



"""
Tests for Learner Recommendations views and related functions.
"""

import json
from unittest import mock

from django.urls import reverse_lazy
from rest_framework.test import APITestCase

from test_utils.constants import mock_cross_product_recommendation_keys
from test_utils.factories import UserFactory
from test_utils.helpers import get_general_recommendations


class TestRecommendationsContextView(APITestCase):
    """
    Tests for the Recommendations Context View
    """

    def setUp(self):
        super().setUp()
        self.user = UserFactory()
        self.password = "test"
        self.url = reverse_lazy("learner_dashboard_recommendations_context")

    @mock.patch("edx_recommendations.api.cross_product_recommendations.country_code_from_ip")
    def test_successful_response(self, country_code_from_ip_mock):
        """
        Test that country code gets sent back for authenticated user
        """

        country_code_from_ip_mock.return_value = "za"
        self.client.login(username=self.user.username, password=self.password)

        response = self.client.get(self.url)
        response_data = json.loads(response.content)

        self.assertEqual(response_data["countryCode"], "za")

    def test_unauthenticated_response(self):
        """
        Test that a 401 is sent back if an unauthenticated user calls endpoint
        """
        response = self.client.get(self.url)

        self.assertEqual(response.status_code, 401)


class TestCrossProductRecommendationsView(APITestCase):
    """
    Tests for the Cross Product Recommendations View
    """

    def setUp(self):
        super().setUp()
        self.associated_course_keys = ["edx+HL1", "edx+HL2"]

    def _get_url(self, course_key):
        """
        Returns the API url
        """
        return reverse_lazy(
            "course_about_page_cross_product",
            kwargs={'course_id': f'course-v1:{course_key}+Test_Course'}
        )

    def _get_recommended_courses(self, num_of_courses_with_restriction=0, active_course_run=True):
        """
        Returns an array of two discovery courses with or without country restrictions
        """
        courses = []
        restriction_obj = {
            "restriction_type": "blocklist",
            "countries": ["CN"],
            "states": []
        }

        for course_key in enumerate(self.associated_course_keys):
            location_restriction = restriction_obj if num_of_courses_with_restriction > 0 else None
            advertised_course_run_uuid = "jh76b2c9-589b-4d1e-88c1-b01a02db3a9c" if active_course_run else None

            courses.append({
                "key": course_key[1],
                "uuid": "6f8cb2c9-589b-4d1e-88c1-b01a02db3a9c",
                "title": f"Title {course_key[0]}",
                "image": {
                        "src": "https://www.logo_image_url.com",
                },
                "url_slug": "https://www.marketing_url.com",
                "course_type": "executive-education",
                "owners": [
                    {
                            "key": "org-1",
                            "name": "org 1",
                            "logo_image_url": "https://discovery.com/organization/logos/org-1.png",
                    },
                ],
                "course_runs": [
                    {
                        "key": "course-v1:Test+2023_T2",
                        "marketing_url": "https://www.marketing_url.com",
                        "availability": "Current",
                        "uuid": "jh76b2c9-589b-4d1e-88c1-b01a02db3a9c",
                        "status": "published"
                    }
                ],
                "advertised_course_run_uuid": advertised_course_run_uuid,
                "location_restriction": location_restriction,
            })

            if num_of_courses_with_restriction > 0:
                num_of_courses_with_restriction -= 1

        return courses

    @mock.patch("django.conf.settings.CROSS_PRODUCT_RECOMMENDATIONS_KEYS", mock_cross_product_recommendation_keys)
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_course_data")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.country_code_from_ip")
    def test_successful_response(
        self, country_code_from_ip_mock, get_course_data_mock,
    ):
        """
        Verify valid cross product course recommendations are returned.
        """
        country_code_from_ip_mock.return_value = "za"
        mock_course_data = self._get_recommended_courses()
        get_course_data_mock.side_effect = [mock_course_data[0], mock_course_data[1]]

        response = self.client.get(self._get_url('edx+HL0'))
        response_content = json.loads(response.content)
        course_data = response_content["courses"]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(course_data), 2)

    @mock.patch("django.conf.settings.CROSS_PRODUCT_RECOMMENDATIONS_KEYS", mock_cross_product_recommendation_keys)
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_course_data")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.country_code_from_ip")
    def test_one_course_country_restriction_response(
        self, country_code_from_ip_mock, get_course_data_mock,
    ):
        """
        Verify valid cross product course recommendation is returned
        if there is a location restriction on a product
        """
        country_code_from_ip_mock.return_value = "cn"
        mock_course_data = self._get_recommended_courses(1)
        get_course_data_mock.side_effect = [mock_course_data[0], mock_course_data[1]]

        response = self.client.get(self._get_url('edx+HL0'))
        response_content = json.loads(response.content)
        course_data = response_content["courses"]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(course_data), 1)
        self.assertEqual(course_data[0]["title"], "Title 1")

    @mock.patch("django.conf.settings.CROSS_PRODUCT_RECOMMENDATIONS_KEYS", mock_cross_product_recommendation_keys)
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_course_data")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.country_code_from_ip")
    def test_both_course_country_restriction_response(
        self, country_code_from_ip_mock, get_course_data_mock,
    ):
        """
        Verify no courses are returned if both courses have a location restriction
        for the users country.
        """
        country_code_from_ip_mock.return_value = "cn"
        mock_course_data = self._get_recommended_courses(2)

        get_course_data_mock.side_effect = [mock_course_data[0], mock_course_data[1]]

        response = self.client.get(self._get_url('edx+HL0'))
        response_content = json.loads(response.content)
        course_data = response_content["courses"]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(course_data), 0)

    @mock.patch("django.conf.settings.CROSS_PRODUCT_RECOMMENDATIONS_KEYS", mock_cross_product_recommendation_keys)
    def test_no_associated_course_response(self):
        """
        Verify an empty array of courses is returned if there are no associated course keys.
        """
        response = self.client.get(self._get_url('No+Associations'))
        response_content = json.loads(response.content)
        course_data = response_content["courses"]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(course_data), 0)

    @mock.patch("django.conf.settings.CROSS_PRODUCT_RECOMMENDATIONS_KEYS", mock_cross_product_recommendation_keys)
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_course_data")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.country_code_from_ip")
    def test_no_response_from_discovery(self, country_code_from_ip_mock, get_course_data_mock):
        """
        Verify an empty array of courses is returned if discovery returns two empty dictionaries.
        """
        country_code_from_ip_mock.return_value = "za"
        get_course_data_mock.side_effect = [{}, {}]

        response = self.client.get(self._get_url('edx+HL0'))
        response_content = json.loads(response.content)
        course_data = response_content["courses"]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(course_data), 0)

    @mock.patch("django.conf.settings.CROSS_PRODUCT_RECOMMENDATIONS_KEYS", mock_cross_product_recommendation_keys)
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_course_data")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.country_code_from_ip")
    def test_no_active_course_runs_response(self, country_code_from_ip_mock, get_course_data_mock):
        """
        Verify that an empty array of courses is returned if courses do not have an active course run.
        """
        country_code_from_ip_mock.return_value = "za"
        mock_course_data = self._get_recommended_courses(0, active_course_run=False)
        get_course_data_mock.side_effect = [mock_course_data[0], mock_course_data[1]]

        response = self.client.get(self._get_url('edx+HL0'))
        reponse_content = json.loads(response.content)
        course_data = reponse_content["courses"]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(course_data), 0)


class TestProductRecommendationsView(APITestCase):
    """
    Tests for Product Recommendations View
    """

    def setUp(self):
        super().setUp()
        self.user = UserFactory()
        self.client.login(username=self.user.username, password="test")
        self.associated_course_keys = ["edx+HL1", "edx+HL2"]
        self.amplitude_keys = [
            "edx+CS0",
            "edx+CS10",
            "edx+CS20",
            "edx+CS30",
            "edx+CS40",
            "edx+CS50",
            "edx+CS60",
            "edx+CS70",
            "edx+CS80",
            "edx+CS90",
        ]
        self.amplitude_course_run_keys = [f"course-v1:{course_key}+2023_T2" for course_key in self.amplitude_keys]
        self.enrolled_course_run_keys = self.amplitude_course_run_keys[3:8]
        self.enrolled_course_keys = self.amplitude_keys[3:8]
        self.amplitude_location_restriction_keys = self.amplitude_keys[0:3]
        self.cross_product_location_restriction_keys = self.associated_course_keys[0]

    def _get_url(self, course_key=None):
        """
        Returns the product recommendations url with or without the course key
        """
        if course_key:
            return reverse_lazy(
                "learner_dashboard_cross_product",
                kwargs={'course_id': f'course-v1:{course_key}+Test_Course'}
            )

        return reverse_lazy(
            "learner_dashboard_amplitude_v2"
        )

    def _get_product_recommendations(self, course_keys, keys_with_restriction=None):
        """
        Returns course data based on the number of course keys passed in
        with a location restriction object if a list of keys for location restriction courses is passed in
        """
        courses = []

        for key in course_keys:
            course = {
                "title": f"Title for {key}",
                "image": {
                    "src": "https://www.logo_image_url.com",
                },
                "course_type": "executive-education",
                "owners": [
                    {
                        "key": "org-1",
                        "name": "org 1",
                        "logo_image_url": "https://discovery.com/organization/logos/org-1.png",
                    },
                ],
                "course_runs": [
                    {
                        "key": f"course-v1:{key}+2023_T2",
                        "marketing_url": "https://www.marketing_url.com",
                        "availability": "Current",
                        "uuid": "jh76b2c9-589b-4d1e-88c1-b01a02db3a9c",
                        "status": "published"
                    }
                ],
                "marketing_url": "https://www.marketing_url.com/course/some-course",
                "advertised_course_run_uuid": f"course-v1:{key}+2023_T2",
            }
            if keys_with_restriction and key in keys_with_restriction:
                course.update({
                    "location_restriction": {
                        "restriction_type": "blocklist",
                        "countries": ["CN"],
                        "states": []
                    }
                })

            courses.append(course)

        return courses

    @mock.patch("django.conf.settings.CROSS_PRODUCT_RECOMMENDATIONS_KEYS", mock_cross_product_recommendation_keys)
    @mock.patch("edx_recommendations.api.utils._get_user_enrolled_course_keys")
    @mock.patch("edx_recommendations.api.utils.get_course_data")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_course_data")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_amplitude_course_recommendations")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.country_code_from_ip")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.is_user_enrolled_in_ut_austin_masters_program")
    def test_successful_response(
        self,
        is_user_enrolled_in_ut_austin_masters_program_mock,
        country_code_from_ip_mock,
        get_amplitude_course_recommendations_mock,
        get_course_data_view_mock,
        get_course_data_util_mock,
        get_user_enrolled_course_keys_mock,
    ):
        """
        Verify 2 cross product course recommendations are returned
        and 4 amplitude courses are returned
        """
        is_user_enrolled_in_ut_austin_masters_program_mock.return_value = False
        country_code_from_ip_mock.return_value = "za"
        get_user_enrolled_course_keys_mock.return_value = []
        get_amplitude_course_recommendations_mock.return_value = [False, True, self.amplitude_keys]

        mock_cross_product_course_data = self._get_product_recommendations(self.associated_course_keys)
        mock_amplitude_course_data = self._get_product_recommendations(self.amplitude_keys)
        get_course_data_view_mock.side_effect = mock_cross_product_course_data
        get_course_data_util_mock.side_effect = mock_amplitude_course_data

        response = self.client.get(self._get_url('edx+HL0'))
        response_content = json.loads(response.content)
        cross_product_course_data = response_content["crossProductCourses"]
        amplitude_course_data = response_content["amplitudeCourses"]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(cross_product_course_data), 2)
        self.assertEqual(len(amplitude_course_data), 4)

    @mock.patch("django.conf.settings.CROSS_PRODUCT_RECOMMENDATIONS_KEYS", mock_cross_product_recommendation_keys)
    @mock.patch("edx_recommendations.api.utils._get_user_enrolled_course_keys")
    @mock.patch("edx_recommendations.api.utils.get_course_data")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_course_data")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_amplitude_course_recommendations")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.country_code_from_ip")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.is_user_enrolled_in_ut_austin_masters_program")
    def test_successful_course_filtering(
        self,
        is_user_enrolled_in_ut_austin_masters_program_mock,
        country_code_from_ip_mock,
        get_amplitude_course_recommendations_mock,
        get_course_data_view_mock,
        get_course_data_util_mock,
        get_user_enrolled_course_keys_mock,
    ):
        """
        Verify 1 cross product course recommendation is returned
        and 2 amplitude courses are returned with filtering done for
        enrolled courses and courses with country restrictions
        """
        is_user_enrolled_in_ut_austin_masters_program_mock.return_value = False
        country_code_from_ip_mock.return_value = "cn"
        get_user_enrolled_course_keys_mock.return_value = self.enrolled_course_run_keys
        get_amplitude_course_recommendations_mock.return_value = [False, True, self.amplitude_keys]

        mock_cross_product_course_data = self._get_product_recommendations(
            self.associated_course_keys, self.cross_product_location_restriction_keys
        )
        mock_amplitude_course_data = self._get_product_recommendations(
            self.amplitude_keys, self.amplitude_location_restriction_keys
        )
        get_course_data_view_mock.side_effect = mock_cross_product_course_data
        get_course_data_util_mock.side_effect = mock_amplitude_course_data

        response = self.client.get(self._get_url('edx+HL0'))
        response_content = json.loads(response.content)
        cross_product_course_data = response_content["crossProductCourses"]
        amplitude_course_data = response_content["amplitudeCourses"]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(cross_product_course_data), 1)
        self.assertEqual(len(amplitude_course_data), 2)
        for course in amplitude_course_data:
            course_key = course["title"][2]
            assert course_key not in [*self.amplitude_location_restriction_keys, *self.enrolled_course_keys]
        for course in cross_product_course_data:
            course_key = course["title"][2]
            assert course_key not in self.cross_product_location_restriction_keys

    @mock.patch("django.conf.settings.CROSS_PRODUCT_RECOMMENDATIONS_KEYS", mock_cross_product_recommendation_keys)
    @mock.patch("django.conf.settings.GENERAL_RECOMMENDATIONS", get_general_recommendations())
    @mock.patch("edx_recommendations.api.utils._get_user_enrolled_course_keys")
    @mock.patch("edx_recommendations.api.utils.get_course_data")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_course_data")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_amplitude_course_recommendations")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.country_code_from_ip")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.is_user_enrolled_in_ut_austin_masters_program")
    def test_fallback_recommendations_when_enrolled_courses_removed(
        self,
        is_user_enrolled_in_ut_austin_masters_program_mock,
        country_code_from_ip_mock,
        get_amplitude_course_recommendations_mock,
        get_course_data_view_mock,
        get_course_data_util_mock,
        get_user_enrolled_course_keys_mock
    ):
        """
        Verify 2 cross product course recommendations are returned
        and 4 fallback amplitude recommendations are returned if no courses are left
        after filtering due to courses being already enrolled in
        """
        is_user_enrolled_in_ut_austin_masters_program_mock.return_value = False
        country_code_from_ip_mock.return_value = "za"
        get_user_enrolled_course_keys_mock.return_value = self.amplitude_course_run_keys
        get_amplitude_course_recommendations_mock.return_value = [False, True, self.amplitude_keys]

        mock_cross_product_course_data = self._get_product_recommendations(self.associated_course_keys)
        mock_amplitude_course_data = self._get_product_recommendations(self.amplitude_keys)
        get_course_data_view_mock.side_effect = mock_cross_product_course_data
        get_course_data_util_mock.side_effect = mock_amplitude_course_data

        response = self.client.get(self._get_url('edx+HL0'))
        response_content = json.loads(response.content)
        cross_product_course_data = response_content["crossProductCourses"]
        amplitude_course_data = response_content["amplitudeCourses"]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(cross_product_course_data), 2)
        self.assertEqual(len(amplitude_course_data), 4)
        for course in amplitude_course_data:
            self.assertEqual(course["title"], "Introduction to Computer Science and Programming Using Python")

    @mock.patch("django.conf.settings.CROSS_PRODUCT_RECOMMENDATIONS_KEYS", mock_cross_product_recommendation_keys)
    @mock.patch("django.conf.settings.GENERAL_RECOMMENDATIONS", get_general_recommendations())
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_course_data")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_amplitude_course_recommendations")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.country_code_from_ip")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.is_user_enrolled_in_ut_austin_masters_program")
    def test_fallback_recommendations_when_error_querying_amplitude(
        self,
        is_user_enrolled_in_ut_austin_masters_program_mock,
        country_code_from_ip_mock,
        get_amplitude_course_recommendations_mock,
        get_course_data_mock,
    ):
        """
        Verify 2 cross product course recommendations are returned
        and 4 fallback amplitude recommendations are returned
        if there was an error querying amplitude for recommendations
        """
        is_user_enrolled_in_ut_austin_masters_program_mock.return_value = False
        country_code_from_ip_mock.return_value = "za"
        get_amplitude_course_recommendations_mock.side_effect = Exception()

        mock_cross_product_course_data = self._get_product_recommendations(self.associated_course_keys)
        get_course_data_mock.side_effect = mock_cross_product_course_data

        response = self.client.get(self._get_url('edx+HL0'))
        response_content = json.loads(response.content)
        cross_product_course_data = response_content["crossProductCourses"]
        amplitude_course_data = response_content["amplitudeCourses"]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(cross_product_course_data), 2)
        self.assertEqual(len(amplitude_course_data), 4)
        for course in amplitude_course_data:
            self.assertEqual(course["title"], "Introduction to Computer Science and Programming Using Python")

    @mock.patch("django.conf.settings.CROSS_PRODUCT_RECOMMENDATIONS_KEYS", mock_cross_product_recommendation_keys)
    @mock.patch("django.conf.settings.GENERAL_RECOMMENDATIONS", get_general_recommendations())
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_course_data")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_amplitude_course_recommendations")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.country_code_from_ip")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.is_user_enrolled_in_ut_austin_masters_program")
    def test_fallback_recommendations_when_no_amplitude_recommended_keys(
        self,
        is_user_enrolled_in_ut_austin_masters_program_mock,
        country_code_from_ip_mock,
        get_amplitude_course_recommendations_mock,
        get_course_data_mock,
    ):
        """
        Verify 2 cross product course recommendations are returned
        and 4 fallback amplitude recommendations are returned
        if amplitude gave back no course keys
        """
        is_user_enrolled_in_ut_austin_masters_program_mock.return_value = False
        country_code_from_ip_mock.return_value = "za"
        get_amplitude_course_recommendations_mock.side_effect = [False, True, []]

        mock_cross_product_course_data = self._get_product_recommendations(self.associated_course_keys)
        get_course_data_mock.side_effect = mock_cross_product_course_data

        response = self.client.get(self._get_url('edx+HL0'))
        response_content = json.loads(response.content)
        cross_product_course_data = response_content["crossProductCourses"]
        amplitude_course_data = response_content["amplitudeCourses"]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(cross_product_course_data), 2)
        self.assertEqual(len(amplitude_course_data), 4)
        for course in amplitude_course_data:
            self.assertEqual(course["title"], "Introduction to Computer Science and Programming Using Python")

    @mock.patch("django.conf.settings.CROSS_PRODUCT_RECOMMENDATIONS_KEYS", mock_cross_product_recommendation_keys)
    @mock.patch("edx_recommendations.api.utils._get_user_enrolled_course_keys")
    @mock.patch("edx_recommendations.api.utils.get_course_data")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_amplitude_course_recommendations")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.country_code_from_ip")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.is_user_enrolled_in_ut_austin_masters_program")
    def test_response_with_amplitude_and_no_cross_product_courses(
        self,
        is_user_enrolled_in_ut_austin_masters_program_mock,
        country_code_from_ip_mock,
        get_amplitude_course_recommendations_mock,
        get_course_data_mock,
        get_user_enrolled_course_keys_mock
    ):
        """
        Verify that if no cross product courses are returned,
        then 4 fallback amplitude recommendations will still be returned
        """
        is_user_enrolled_in_ut_austin_masters_program_mock.return_value = False
        country_code_from_ip_mock.return_value = "za"
        get_user_enrolled_course_keys_mock.return_value = self.enrolled_course_run_keys
        get_amplitude_course_recommendations_mock.return_value = [False, True, self.amplitude_keys]

        mock_amplitude_course_data = self._get_product_recommendations(self.amplitude_keys)
        get_course_data_mock.side_effect = mock_amplitude_course_data

        response = self.client.get(self._get_url('No+Association'))
        response_content = json.loads(response.content)
        cross_product_course_data = response_content["crossProductCourses"]
        amplitude_course_data = response_content["amplitudeCourses"]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(cross_product_course_data), 0)
        self.assertEqual(len(amplitude_course_data), 4)

    @mock.patch("edx_recommendations.api.utils._get_user_enrolled_course_keys")
    @mock.patch("edx_recommendations.api.utils.get_course_data")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.get_amplitude_course_recommendations")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.country_code_from_ip")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.is_user_enrolled_in_ut_austin_masters_program")
    def test_amplitude_only_url_response(
        self,
        is_user_enrolled_in_ut_austin_masters_program_mock,
        country_code_from_ip_mock,
        get_amplitude_course_recommendations_mock,
        get_course_data_mock,
        get_user_enrolled_course_keys_mock
    ):
        """
        Verify that if no course key was provided in the url,
        only 1 field for amplitude courses are sent back
        """
        is_user_enrolled_in_ut_austin_masters_program_mock.return_value = False
        country_code_from_ip_mock.return_value = "za"
        get_user_enrolled_course_keys_mock.return_value = self.enrolled_course_run_keys
        get_amplitude_course_recommendations_mock.return_value = [False, True, self.amplitude_keys]

        mock_amplitude_course_data = self._get_product_recommendations(self.amplitude_keys)
        get_course_data_mock.side_effect = mock_amplitude_course_data

        response = self.client.get(self._get_url())
        response_content = json.loads(response.content)
        amplitude_course_data = response_content["amplitudeCourses"]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response_content), 1)
        self.assertEqual(len(amplitude_course_data), 4)

    @mock.patch("edx_recommendations.api.cross_product_recommendations.country_code_from_ip")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.is_user_enrolled_in_ut_austin_masters_program")
    def test_zero_cross_product_and_amplitude_recommendations(
        self,
        is_user_enrolled_in_ut_austin_masters_program_mock,
        country_code_from_ip_mock,
    ):
        """
        Verify 0 cross product course recommendations are returned
        and 0 amplitude courses are returned if the user is enrolled in ut austin masters program
        """
        is_user_enrolled_in_ut_austin_masters_program_mock.return_value = True
        country_code_from_ip_mock.return_value = "za"

        response = self.client.get(self._get_url('edx+HL0'))
        response_content = json.loads(response.content)
        cross_product_course_data = response_content["crossProductCourses"]
        amplitude_course_data = response_content["amplitudeCourses"]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(cross_product_course_data), 0)
        self.assertEqual(len(amplitude_course_data), 0)

    @mock.patch("edx_recommendations.api.cross_product_recommendations.country_code_from_ip")
    @mock.patch("edx_recommendations.api.cross_product_recommendations.is_user_enrolled_in_ut_austin_masters_program")
    def test_zero_amplitude_recommendations(
        self,
        is_user_enrolled_in_ut_austin_masters_program_mock,
        country_code_from_ip_mock,
    ):
        """
        Verify that 0 amplitude courses are returned
        if the user is enrolled in ut austin masters program
        """
        is_user_enrolled_in_ut_austin_masters_program_mock.return_value = True
        country_code_from_ip_mock.return_value = "za"

        response = self.client.get(self._get_url())
        response_content = json.loads(response.content)
        amplitude_course_data = response_content["amplitudeCourses"]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(amplitude_course_data), 0)
